use std::io;

fn main() {
    let mut input = String::new();

    io::stdin()
        .read_line(&mut input)
        .expect("Failed to read line");

    let [n]: [u32; 1] = input
        .trim()
        .split(" ")
        .map(|x| x.trim().parse().unwrap())
        .collect::<Vec<_>>()
        .try_into()
        .unwrap();

    let mut graph = Graph::new(n);
    loop {
        input = String::new();

        if let Err(_) = io::stdin().read_line(&mut input) {
            break
        }

        if input.trim().is_empty() {
            break
        }

        let [a, b]: [u32; 2] = input
            .trim()
            .split(" ")
            //.inspect(|x| println!("\"{x}\""))
            .map(|x| x.trim().parse().unwrap())
            .collect::<Vec<_>>()
            .try_into()
            .unwrap();
        
        graph.add_conn(a, b);
    }

    let paths = graph.calculate_longest_paths();

    //println!("{paths:?}");

    let test: Vec<_> = paths
        .iter()
        .enumerate()
        .map(|(x1, y1)| paths.iter().enumerate().map(move |(x2, y2)| ((x1, x2),(y1, y2))))
        .flatten()
        .map(|(x, (y, z))| (x, common_items(y, z)))
        .collect();

    let min = test.iter().map(|(_, y)| y).min().unwrap();

    let (a, b) = test
        .iter()
        .filter(|(_, y)| y == min)
        .map(|((x, y), _)| (paths[*x].clone(), paths[*y].clone()))
        .next()
        .unwrap();

    println!("{}", paths[0].len());
    for i in a {
        print!("{i} ");
    }
    println!("");
    for i in b {
        print!("{i} ");
    }
}

fn common_items<T: PartialEq>(list1: &[T], list2: &[T]) -> usize {
    list1
        .iter()
        .map(|x| list2.iter().filter(|&y| x == y).count())
        .sum::<usize>()
}

#[derive(Debug)]
struct Graph {
    nodes: Vec<u32>,
    connections: Vec<(u32, u32)>,
}

impl Graph {
    fn new(node_amount: u32) -> Self {
        Self {
            nodes: (1..=node_amount).collect(),
            connections: Vec::new()
        }
    }

    fn add_conn(&mut self, from: u32, to: u32) {
        self.connections.push((from, to))
    }

    fn connections_of_node(&self, node_num: u32) -> Vec<u32> {
        self.connections.iter().filter_map(|&(x, y)|
            if x == node_num { Some(y) }
            else if y == node_num { Some(x) }
            else { None })
        .collect()
    }

    fn calculate_longest_path(&self) -> Vec<u32> {
        self
            .nodes
            .iter()
            .filter(|&&x| self.connections_of_node(x).len() == 1)
            .map(|&x| self.find_longest_paths_from_node(x, None).into_iter())
            .flatten()
            .max_by(|x, y| x.len().cmp(&y.len()))
            .unwrap()
    }

    fn calculate_longest_paths(&self) -> Vec<Vec<u32>> {
        let max_len = self.calculate_longest_path().len();

        self
            .nodes
            .iter()
            //.inspect(|x| println!("Node {x:?}, conns: {:?}", self.connections_of_node(**x)))
            .filter(|&&x| self.connections_of_node(x).len() == 1)
            .map(|&x| self.find_longest_paths_from_node(x, None).into_iter())
            .flatten()
            //.inspect(|x| println!("{x:?}"))
            .filter(|x| x.len() == max_len)
            .collect()
    }
 
    fn find_longest_paths_from_node(&self, node: u32, prev_node: Option<u32>)-> Vec<Vec<u32>> {
        let tmp: Vec<_> = self
            .connections_of_node(node)
            .iter()
            .filter(|&x| *x != prev_node.unwrap_or(x+1))
            .map(|&x| self.find_longest_paths_from_node(x, Some(node)).into_iter())
            .flatten()
            .collect();

        let max_len = tmp.iter()
            .map(|x| x.len())
            .max()
            .unwrap_or(0);

        if max_len == 0{ 
            return vec![vec![node]]
        }

        tmp.iter()
            //.inspect(|x| println!("{x:?} {}", x.len()))
            .filter(|x| x.len() == max_len)
            .map(|x| std::iter::once(&node).chain(x.into_iter()).map(|&x| x).collect())
            .collect()
    }
}