#include <bits/stdc++.h>
using namespace std;

struct Tree {
    int n;
    vector<vector<int>> edges{};
    vector<int> parents{};
    vector<int> jumps{};
    vector<int> st_roots{};
    vector<int> depths{};
    vector<int> max_depths{};

    Tree(int n) {
        this->n = n;
        this->edges.resize(n);
        this->parents.resize(n);
        this->jumps.resize(n);
        this->st_roots.resize(n);
        this->depths.resize(n);
        this->max_depths.resize(n);
    }

    void add_edge(int u, int v) {
        edges[u].push_back(v);
        edges[v].push_back(u);
    }

    void prepare_tree(int from = 0) {
        _prepare_tree(from, -1, -1, 0);
    }

    void _prepare_tree(int from, int par, int st_root, int depth) {
        parents[from] = par;
        st_roots[from] = st_root;
        depths[from] = depth;
        max_depths[from] = depth;

        if (par == -1) {
            jumps[from] = from;
        } else {
            if (depths[jumps[par]] - depths[par] == depths[jumps[jumps[par]]] - depths[jumps[par]]) {
                jumps[from] = jumps[jumps[par]];
            } else {
                jumps[from] = par;
            }
        }

        for (int to : edges[from]) {
            if (to == par) {
                continue;
            }

            _prepare_tree(to, from, st_root >= 0 ? st_root : to, depth + 1);
            
            if (max_depths[to] > max_depths[from]) {
                max_depths[from] = max_depths[to];
            }
        }
    }

    int get_center(int from = 0) {
        return _get_center(from, -1, 0);
    }

    int _get_center(int from, int par, int par_depth) {
        int max_depth = 0;
        int max_depth_vertex = -1;
        int second_max_depth = 0;

        for (int to : edges[from]) {
            int depth = to == par ? par_depth : max_depths[to] - depths[from];

            if (depth >= max_depth) {
                second_max_depth = max_depth;
                max_depth = depth;
                max_depth_vertex = to;
            } else if (depth >= second_max_depth) {
                second_max_depth = depth;
            }
        }

        return max_depth_vertex == par || second_max_depth == max_depth
            ? from
            : _get_center(max_depth_vertex, from, second_max_depth + 1);
    }

    int get_kth_parent(int v, int k) {
        int target_depth = depths[v] - k;

        if (target_depth < 0) {
            return -1;
        }

        while (depths[v] != target_depth) {
            if (depths[jumps[v]] >= target_depth) {
                v = jumps[v];
            } else {
                v = parents[v];
            }
        }

        return v;
    }

    int get_lca(int u, int v) {
        if (depths[u] < depths[v]) {
            swap(u, v);
        }

        u = get_kth_parent(u, depths[u] - depths[v]);

        while (u != v) {
            if (jumps[u] == jumps[v]) {
                u = parents[u];
                v = parents[v];
            } else {
                u = jumps[u];
                v = jumps[v];
            }
        }

        return u;
    }

    tuple<int, int, int> get_least_connected_vertices(const vector<int>& options) {
        int vertex = options[0];
        int lca = vertex;

        for (int option : options) {
            int new_lca = get_lca(lca, option);

            if (new_lca != lca) {
                lca = new_lca;
                vertex = option;
            }
        }

        return { options[0], vertex, lca };
    }
};

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
    
    int n;
    cin >> n;

    Tree tree(n);

    for (int i = 0; i < n - 1; i++) {
        int u, v;
        cin >> u >> v;

        tree.add_edge(u - 1, v - 1);
    }

    tree.prepare_tree();

    int center = tree.get_center();

    tree.prepare_tree(center);

    map<int, map<int, vector<int>>, greater<int>> vertices_by_depth{};
    vertices_by_depth[0][-1].push_back(center);

    for (int to = 0; to < n; to++) {
        if (tree.depths[to] == tree.max_depths[tree.st_roots[to]] && to != center) {
            vertices_by_depth[tree.depths[to]][tree.st_roots[to]].push_back(to);
        }
    }

    int path1_start = -1;
    int path1_end = -1;
    int path2_start = -1;
    int path2_end = -1;

    auto [max_depth1, max_depth_children1] = *vertices_by_depth.begin();
    auto [max_depth2, max_depth_children2] = *next(vertices_by_depth.begin());

    if (max_depth_children1.size() >= 4) {
        path1_start = (*max_depth_children1.begin()).second[0];
        path1_end = (*next(max_depth_children1.begin())).second[0];
        path2_start = (*next(max_depth_children1.begin(), 2)).second[0];
        path2_end = (*next(max_depth_children1.begin(), 3)).second[0];
    } else if (max_depth_children1.size() == 3) {
        vector<int> options[3] = {
            (*max_depth_children1.begin()).second,
            (*next(max_depth_children1.begin())).second,
            (*next(max_depth_children1.begin(), 2)).second,
        };

        int best = n + 1;

        for (int i = 0; i < 3; i++) {
            const vector<int>& curr = options[i];
            const vector<int>& other1 = options[(i + 1) % 3];
            const vector<int>& other2 = options[(i + 2) % 3];

            auto [vertex1, vertex2, lca] = tree.get_least_connected_vertices(curr);

            if (tree.depths[lca] < best) {
                best = tree.depths[lca];
                path1_start = other1[0];
                path1_end = vertex1;
                path2_start = vertex2;
                path2_end = other2[0];
            }
        }
    } else if (max_depth_children1.size() == 2) {
        auto [v1, v2, lca1] = tree.get_least_connected_vertices((*max_depth_children1.begin()).second);
        auto [u1, u2, lca2] = tree.get_least_connected_vertices((*next(max_depth_children1.begin())).second);
        path1_start = v1;
        path1_end = u1;
        path2_start = v2;
        path2_end = u2;
    } else if (max_depth_children2.size() >= 2) {
        auto [vertex1, vertex2, lca] = tree.get_least_connected_vertices((*max_depth_children1.begin()).second);
        path1_start = vertex1;
        path2_start = vertex2;
        path1_end = (*max_depth_children2.begin()).second[0];
        path2_end = (*next(max_depth_children2.begin())).second[0];
    } else {
        auto [v1, v2, lca1] = tree.get_least_connected_vertices((*max_depth_children1.begin()).second);
        auto [u1, u2, lca2] = tree.get_least_connected_vertices((*max_depth_children2.begin()).second);
        path1_start = v1;
        path1_end = u1;
        path2_start = v2;
        path2_end = u2;
    }

    // cout << center << "\n";
    // cout << max_depth1 << " " << max_depth_children1.size() << "\n";
    // cout << max_depth2 << " " << max_depth_children2.size() << "\n";

    vector<int> path1_beginning{};

    int curr = path1_start;

    while (curr != -1) {
        path1_beginning.push_back(curr);
        curr = tree.parents[curr];
    }

    vector<int> path1_ending{};

    curr = path1_end;

    while (curr != center) {
        path1_ending.push_back(curr);
        curr = tree.parents[curr];
    }

    reverse(path1_ending.begin(), path1_ending.end());

    cout << path1_beginning.size() + path1_ending.size() << "\n";

    for (int v : path1_beginning) {
        cout << v + 1 << " ";
    }

    for (int v : path1_ending) {
        cout << v + 1 << " ";
    }

    cout << "\n";

    curr = path2_start;

    while (curr != -1) {
        cout << curr + 1 << " ";
        curr = tree.parents[curr];
    }

    vector<int> path2_ending{};

    curr = path2_end;

    while (curr != center) {
        path2_ending.push_back(curr);
        curr = tree.parents[curr];
    }

    reverse(path2_ending.begin(), path2_ending.end());

    for (int v : path2_ending) {
        cout << v + 1 << " ";
    }

    return 0;
}
